import os

import tensorflow as tf
import tensorflow_hub as hub
import tensorflow_transform as tft
from census_consumer_complaint_component.feature_engineering.feature_engineering import (
    BUCKET_FEATURES,
    LABEL_KEY,
    ONE_HOT_FEATURES,
    TEXT_FEATURES,
    transformed_name,
)


def get_model(show_summary=True):
    """
    This function defines a Keras model and returns the model as a
    Keras object.
    """

    # one-hot categorical features
    input_features = []
    for key, dim in ONE_HOT_FEATURES.items():
        input_features.append(
            tf.keras.Input(shape=(dim + 1,), name=transformed_name(key))
        )

    # # adding bucketized features
    # for key, dim in BUCKET_FEATURES.items():
    #     input_features.append(
    #         tf.keras.Input(shape=(dim + 1,), name=transformed_name(key))
    #     )

    # adding text input features
    # input_texts = []
    # for key in TEXT_FEATURES.keys():
    #     input_texts.append(
    #         tf.keras.Input(
    #             shape=(1,), name=transformed_name(key), dtype=tf.string
    #         )
    #     )
    #
    # # embed text features
    # MODULE_URL = "https://tfhub.dev/google/universal-sentence-encoder/4"
    # embed = hub.KerasLayer(MODULE_URL)
    # reshaped_narrative = tf.reshape(input_texts[0], [-1])
    # embed_narrative = embed(reshaped_narrative)
    # deep_ff = tf.keras.layers.Reshape((512,), input_shape=(1, 512))(
    #     embed_narrative
    # )
    #
    # deep = tf.keras.layers.Dense(256, activation="relu")(deep_ff)
    # deep = tf.keras.layers.Dense(64, activation="relu")(deep)
    # deep = tf.keras.layers.Dense(16, activation="relu")(deep)

    wide_ff = tf.keras.layers.concatenate(input_features)
    wide = tf.keras.layers.Dense(16, activation="relu")(wide_ff)
    wide = tf.keras.layers.Dense(8, activation="relu")(wide)

    # both = tf.keras.layers.concatenate([deep, wide])

    output = tf.keras.layers.Dense(1, activation="sigmoid")(wide)

    inputs = input_features  # + input_texts

    keras_model = tf.keras.models.Model(inputs, output)
    keras_model.compile(
        optimizer=tf.keras.optimizers.Adam(learning_rate=0.001),
        loss="binary_crossentropy",
        metrics=[
            tf.keras.metrics.BinaryAccuracy(),
            tf.keras.metrics.TruePositives(),
        ],
    )
    if show_summary:
        keras_model.summary()

    return keras_model


def _gzip_reader_fn(filenames):
    """Small utility returning a record reader that can read gzip'ed files."""
    return tf.data.TFRecordDataset(filenames, compression_type="GZIP")


def _get_serve_tf_examples_fn(model, tf_transform_output):
    """Returns a function that parses a serialized tf.Example."""

    model.tft_layer = tf_transform_output.transform_features_layer()

    @tf.function
    def serve_tf_examples_fn(serialized_tf_examples):
        """Returns the output to be used in the serving signature."""
        feature_spec = tf_transform_output.raw_feature_spec()
        feature_spec.pop(LABEL_KEY)
        parsed_features = tf.io.parse_example(
            serialized_tf_examples, feature_spec
        )

        transformed_features = model.tft_layer(parsed_features)

        outputs = model(transformed_features)
        return {"outputs": outputs}

    return serve_tf_examples_fn


def _input_fn(file_pattern, tf_transform_output, batch_size=64):
    """Generates features and label for tuning/training.
    Args:
        file_pattern: input tfrecord file pattern.
        tf_transform_output: A TFTransformOutput.
        batch_size: representing the number of consecutive elements of
          returned dataset to combine in a single batch
    Returns:
        A dataset that contains (features, indices) tuple where features
        is a dictionary of Tensors, and indices is a single Tensor of
        label indices.
    """
    transformed_feature_spec = (
        tf_transform_output.transformed_feature_spec().copy()
    )

    dataset = tf.data.experimental.make_batched_features_dataset(
        file_pattern=file_pattern,
        batch_size=batch_size,
        features=transformed_feature_spec,
        reader=_gzip_reader_fn,
        label_key=transformed_name(LABEL_KEY),
    )

    return dataset


# TFX Trainer will call this function.
def run_fn(fn_args):
    """Train the model based on given args.
    Args:
    fn_args: Holds args used to train the model as name/value pairs.
    """
    tf_transform_output = tft.TFTransformOutput(fn_args.transform_output)

    train_dataset = _input_fn(fn_args.train_files, tf_transform_output, 64)
    eval_dataset = _input_fn(fn_args.eval_files, tf_transform_output, 64)

    model = get_model()

    log_dir = os.path.join(os.path.dirname(fn_args.serving_model_dir), "logs")
    tensorboard_callback = tf.keras.callbacks.TensorBoard(
        log_dir=log_dir, update_freq="batch"
    )

    model.fit(
        train_dataset,
        steps_per_epoch=fn_args.train_steps,
        validation_data=eval_dataset,
        validation_steps=fn_args.eval_steps,
        callbacks=[tensorboard_callback],
    )

    signatures = {
        "serving_default": _get_serve_tf_examples_fn(
            model, tf_transform_output
        ).get_concrete_function(
            tf.TensorSpec(shape=[None], dtype=tf.string, name="examples")
        ),
    }
    model.save(
        fn_args.serving_model_dir, save_format="tf", signatures=signatures
    )



from tfx.proto import example_gen_pb2

from tfx.proto import range_config_pb2

from tfx.types import standard_artifacts, standard_component_specs
from tfx.types.component_spec import ChannelParameter
from tfx.types.component_spec import ComponentSpec
from tfx.types.component_spec import ExecutionParameter

REMOTE_ZIP_FILE_URI_KEY = "zip_file_uri"


class RemoteZipFileBasedExampleGenSpec(ComponentSpec):
    """File-based ExampleGen component spec."""

    PARAMETERS = {
        standard_component_specs.INPUT_BASE_KEY:
            ExecutionParameter(type=str),
        REMOTE_ZIP_FILE_URI_KEY:
            ExecutionParameter(type=str),
        standard_component_specs.INPUT_CONFIG_KEY:
            ExecutionParameter(type=example_gen_pb2.Input),
        standard_component_specs.OUTPUT_CONFIG_KEY:
            ExecutionParameter(type=example_gen_pb2.Output),
        standard_component_specs.OUTPUT_DATA_FORMAT_KEY:
            ExecutionParameter(type=int),  # example_gen_pb2.PayloadFormat enum.
        standard_component_specs.OUTPUT_FILE_FORMAT_KEY:
            ExecutionParameter(type=int),  # example_gen_pb2.FileFormat enum.
        standard_component_specs.CUSTOM_CONFIG_KEY:
            ExecutionParameter(type=example_gen_pb2.CustomConfig, optional=True),
        standard_component_specs.RANGE_CONFIG_KEY:
            ExecutionParameter(type=range_config_pb2.RangeConfig, optional=True),
    }
    INPUTS = {}
    OUTPUTS = {
        standard_component_specs.EXAMPLES_KEY: ChannelParameter(type=standard_artifacts.Examples),
    }



import sys
import os
from census_consumer_complaint_exception.exception import CensusConsumerException

from tfx.components import SchemaGen, StatisticsGen, ExampleValidator

from census_consumer_complaint_custom_component.component import RemoteZipFileBasedExampleGen
from census_consumer_complaint_component.data_ingestion import DataIngestion
from collections import namedtuple

DataValidation = namedtuple("DataValidation", ["statistic_gen", "schema_gen", "example_val"])


def get_data_validation_components(data_ingestion: DataIngestion) -> DataValidation:
    """
    :param zip_example_gen:
    :param self:
    :return: List of tfx component
    """
    try:

        data_validation_components = []
        statistic_gen = StatisticsGen(
            examples=data_ingestion.zip_example_gen.outputs['examples']
        )

        schema_gen = SchemaGen(
            statistics=statistic_gen.outputs['statistics']
        )

        example_val = ExampleValidator(schema=schema_gen.outputs['schema'],
                                       statistics=statistic_gen.outputs['statistics'])

        return DataValidation(statistic_gen=statistic_gen,
                              schema_gen=schema_gen,
                              example_val=example_val
                              )

    except Exception as e:
        raise CensusConsumerException(e, sys) from e



# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""TFX RemoteZipCsvExampleGen component definition."""

from typing import Optional, Union

from census_consumer_complaint_custom_component.example_gen.remote_zip_csv_example_gen import executor
from census_consumer_complaint_custom_component.component import RemoteZipFileBasedExampleGen
from tfx.dsl.components.base import executor_spec
from tfx.orchestration import data_types
from tfx.proto import example_gen_pb2
from tfx.proto import range_config_pb2


class RemoteZipCsvExampleGen(RemoteZipFileBasedExampleGen):  # pylint: disable=protected-access
    """Official TFX RemoteZipCsvExampleGen component.

  The remotezipcsv examplegen component takes zip file url of zip compressed csv data, and generates train
  and eval examples for downstream components.

  The remotezipcsv examplegen encodes column values to tf.Example int/float/byte feature.
  For the case when there's missing cells, the csv examplegen uses:
  -- tf.train.Feature(`type`_list=tf.train.`type`List(value=[])), when the
     `type` can be inferred.
  -- tf.train.Feature() when it cannot infer the `type` from the column.

  Note that the type inferring will be per input split. If input isn't a single
  split, users need to ensure the column types align in each pre-splits.

  For example, given the following csv rows of a split:

    header:A,B,C,D
    row1:  1,,x,0.1
    row2:  2,,y,0.2
    row3:  3,,,0.3
    row4:

  The output example will be
    example1: 1(int), empty feature(no type), x(string), 0.1(float)
    example2: 2(int), empty feature(no type), x(string), 0.2(float)
    example3: 3(int), empty feature(no type), empty list(string), 0.3(float)

    Note that the empty feature is `tf.train.Feature()` while empty list string
    feature is `tf.train.Feature(bytes_list=tf.train.BytesList(value=[]))`.

  Component `outputs` contains:
   - `examples`: Channel of type `standard_artifacts.Examples` for output train
                 and eval examples.
  """

    EXECUTOR_SPEC = executor_spec.BeamExecutorSpec(executor.Executor)

    def __init__(
            self,
            input_base: Optional[str] = None,
            zip_file_uri: Optional[str] = None,
            input_config: Optional[Union[example_gen_pb2.Input,
                                         data_types.RuntimeParameter]] = None,
            output_config: Optional[Union[example_gen_pb2.Output,
                                          data_types.RuntimeParameter]] = None,
            range_config: Optional[Union[range_config_pb2.RangeConfig,
                                         data_types.RuntimeParameter]] = None):
        """Construct a RemoteZipCsvExampleGen component.

    Args:
      input_base: an extract directory containing the CSV files after extraction of downloaded zip file.
      zip_file_uri: Remote Zip file uri to download compressed zip csv file
      input_config: An example_gen_pb2.Input instance, providing input
        configuration. If unset, the files under input_base will be treated as a
        single split.
      output_config: An example_gen_pb2.Output instance, providing output
        configuration. If unset, default splits will be 'train' and 'eval' with
        size 2:1.
      range_config: An optional range_config_pb2.RangeConfig instance,
        specifying the range of span values to consider. If unset, driver will
        default to searching for latest span with no restrictions.
    """
        super().__init__(
            input_base=input_base,
            zip_file_uri=zip_file_uri,
            input_config=input_config,
            output_config=output_config,
            range_config=range_config)



import logging
from tfx.orchestration import pipeline
from tfx.orchestration.beam.beam_dag_runner import BeamDagRunner
from tfx.orchestration.metadata import sqlite_metadata_connection_config
from census_consumer_complaint_config.configuration import CensusConsumerConfiguration
from census_consumer_complaint_component.component import get_census_consumer_complaint_pipeline_component

census_consumer_config = CensusConsumerConfiguration()


PIPELINE_ENABLE_CACHE_STATUS = True


# Airflow-specific configs; these will be passed directly to airflow
def _create_pipeline():
    return pipeline.Pipeline(
        pipeline_name=census_consumer_config.pipeline_name,
        pipeline_root=census_consumer_config.pipeline_root,
        components=get_census_consumer_complaint_pipeline_component(),
        enable_cache=PIPELINE_ENABLE_CACHE_STATUS,
        metadata_connection_config=sqlite_metadata_connection_config(census_consumer_config.metadata_path),

    )


def run_apache_dag_pipeline():
    beam_pipeline = BeamDagRunner().run(_create_pipeline())




# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Remote Zip csv based TFX example gen executor."""
import datetime
import os
from typing import Any, Dict, Union
from tfx.dsl.io import fileio
from tfx.components.example_gen.csv_example_gen.executor import _CsvToExample
import pandas as pd
from absl import logging
import apache_beam as beam
import tensorflow as tf
from census_consumer_complaint_utils.utils import download_dataset, extract_zip_file

from tfx.components.example_gen.base_example_gen_executor import BaseExampleGenExecutor
from tfx.types import standard_component_specs
from census_consumer_complaint_types.types import REMOTE_ZIP_FILE_URI_KEY

import pandas as pd


@beam.ptransform_fn
@beam.typehints.with_input_types(beam.Pipeline)
@beam.typehints.with_output_types(tf.train.Example)
def _ZipToExample(  # pylint: disable=invalid-name
        pipeline: beam.Pipeline, exec_properties: Dict[str, Any],
        split_pattern: str) -> beam.pvalue.PCollection:
    """Read remote zip csv files and transform to TF examples.

  Note that each input split will be transformed by this function separately.

  Args:
    pipeline: beam pipeline.
    exec_properties: A dict of execution properties.
      - input_base: input dir that contains Avro data.
    split_pattern: Split.pattern in Input config, glob relative file pattern
      that maps to input files with root directory given by input_base.

  Returns:
    PCollection of TF examples.
  """
    # directory to extract zip file

    input_base_uri = os.path.join(exec_properties[standard_component_specs.INPUT_BASE_KEY])

    # remote zip file uri to download zip file
    zip_file_uri = exec_properties[REMOTE_ZIP_FILE_URI_KEY]

    # downloading zip file from zip file uri into input_base_uri location
    zip_file_path = download_dataset(zip_file_uri, input_base_uri)
    extract_zip_file(zip_file_path, input_base_uri)
    os.remove(zip_file_path)

    for file in os.listdir(input_base_uri):
        csv_file_path = os.path.join(input_base_uri, file)
        df = pd.read_csv(csv_file_path, chunksize=200000)
        file_number = 1
        for data_set in df:
            data_set.dropna(inplace=True, subset=['Consumer disputed?'], axis=0)
            data_set['Consumer disputed?'] = data_set['Consumer disputed?'].replace({'Yes': 1.0, 'No': 0.0})
            data_set.to_csv(os.path.join(input_base_uri, f"file_{file_number}_{file}"), index=None, header=True)
            file_number += 1
            break

        os.remove(csv_file_path)

    return _CsvToExample(exec_properties=exec_properties, split_pattern=split_pattern).expand(pipeline=pipeline)


class Executor(BaseExampleGenExecutor):
    """TFX example gen executor for processing remote zip csv format.

  Data type conversion:
    integer types will be converted to tf.train.Feature with tf.train.Int64List.
    float types will be converted to tf.train.Feature with tf.train.FloatList.
    string types will be converted to tf.train.Feature with tf.train.BytesList
      and utf-8 encoding.

    Note that,
      Single value will be converted to a list of that single value.
      Missing value will be converted to empty tf.train.Feature().

    For details, check the dict_to_example function in example_gen.utils.


  Example usage:

    from tfx.components.base import executor_spec
    from tfx.components.example_gen.component import
    FileBasedExampleGen
    from tfx.components.example_gen.custom_executors import
    avro_executor

    example_gen = FileBasedExampleGen(
        input_base=avro_dir_path,
        custom_executor_spec=executor_spec.ExecutorClassSpec(
            avro_executor.Executor))
  """

    def GetInputSourceToExamplePTransform(self) -> beam.PTransform:
        """Returns PTransform for avro to TF examples."""
        return _ZipToExample



import sys
import os
from census_consumer_complaint_exception.exception import CensusConsumerException
from census_consumer_complaint_config.configuration import CensusConsumerConfiguration
from tfx.components import SchemaGen, StatisticsGen, Transform
from census_consumer_complaint_component.data_ingestion import DataIngestion
from census_consumer_complaint_component.data_validation import DataValidation

from collections import namedtuple

DataPreprocessing = namedtuple("DataPreprocessing", ["transformer"])

census_consumer_config = CensusConsumerConfiguration()


def get_data_preprocessing_components(data_ingestion: DataIngestion,
                                      data_validation: DataValidation
                                      ) -> DataPreprocessing:
    try:
        data_preprocessing_components = []
        transform = Transform(
            examples=data_ingestion.zip_example_gen.outputs['examples'],
            schema=data_validation.schema_gen.outputs['schema'],
            module_file=census_consumer_config.transform_module_file_path
        )
        return DataPreprocessing(transformer=transform)

    except Exception as e:
        raise CensusConsumerException(e, sys) from e



import os


def error_message_detail(error, error_detail):
    _, _, exc_tb = error_detail.exc_info()
    file_name = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
    error_message = "Error occurred python script name [{0}] line number [{1}] error message [{2}]".format(file_name,
                                                                                                           exc_tb.tb_lineno,
                                                                                                           str(error))

    return error_message


class CensusConsumerException(Exception):

    def __init__(self, error_message, error_detail):
        """
        :param error_message: error message in string format
        """
        super().__init__(error_message)
        self.error_message = error_message_detail(error_message, error_detail=error_detail)

    def __repr__(self):
        return CensusConsumerException.__name__.__str__()

    def __str__(self):
        return self.error_message



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import logging
from tfx.orchestration import pipeline
from tfx.orchestration.airflow.airflow_runner import AirflowDAGRunner
from tfx.orchestration.metadata import sqlite_metadata_connection_config
from census_consumer_complaint_config.configuration import CensusConsumerConfiguration

from census_consumer_complaint_component.component import get_census_consumer_complaint_pipeline_component

AIRFLOW_CONFIG_SCHEDULE_INTERVAL_KEY = "schedule_interval"
AIRFLOW_CONFIG_START_DATE_KEY = "start_date"
LOG_OVERRIDES_LOG_ROOT_KEY = "log_root"
LOG_OVERRIDES_LOG_LEVEL_KEY = "log_level"
PIPELINE_ADDITIONAL_PIPELINE_ARGS_KEY = "logger_args"
PIPELINE_ENABLE_CACHE_STATUS = False


census_consumer_config = CensusConsumerConfiguration()

# Airflow-specific configs; these will be passed directly to airflow

_airflow_config = {
    AIRFLOW_CONFIG_SCHEDULE_INTERVAL_KEY: census_consumer_config.scheduled_interval,
    AIRFLOW_CONFIG_START_DATE_KEY: census_consumer_config.start_date,

}

# Logging overrides
logger_overrides = {LOG_OVERRIDES_LOG_ROOT_KEY: census_consumer_config.log_dir,
                    LOG_OVERRIDES_LOG_LEVEL_KEY: logging.INFO}


def _create_pipeline():
    return pipeline.Pipeline(
        pipeline_name=census_consumer_config.pipeline_name,
        pipeline_root=census_consumer_config.pipeline_root,
        components=get_census_consumer_complaint_pipeline_component(),
        enable_cache=PIPELINE_ENABLE_CACHE_STATUS,
        metadata_connection_config=sqlite_metadata_connection_config(census_consumer_config.metadata_path),
        additional_pipeline_args={PIPELINE_ADDITIONAL_PIPELINE_ARGS_KEY: logger_overrides},
    )


def get_airflow_dag_pipeline():
    airflow_pipeline = AirflowDAGRunner(_airflow_config).run(_create_pipeline())
    return airflow_pipeline



from tfx.components import Trainer
from tfx.components.base import executor_spec
from tfx.components.trainer.executor import GenericExecutor
from census_consumer_complaint_exception.exception import CensusConsumerException
from census_consumer_complaint_component.data_ingestion import DataIngestion
from census_consumer_complaint_component.data_validation import DataValidation
from census_consumer_complaint_component.data_preprocessing import DataPreprocessing
from census_consumer_complaint_config.configuration import CensusConsumerConfiguration
from tfx.proto import trainer_pb2
from collections import namedtuple
import sys

TRAINING_STEPS = 1000
EVALUATION_STEPS = 100

ModelTrainer = namedtuple("ModelTrainer", ["trainer"])
census_consumer_config = CensusConsumerConfiguration()


def get_model_trainer_component(data_validation: DataValidation,
                                data_preprocessing: DataPreprocessing):
    try:

        training_kwargs = {
            "module_file": census_consumer_config.trainer_module_file_path,
            "examples": data_preprocessing.transformer.outputs['transformed_examples'],
            "schema": data_validation.schema_gen.outputs['schema'],
            "transform_graph": data_preprocessing.transformer.outputs['transform_graph'],
            "train_args": trainer_pb2.TrainArgs(num_steps=TRAINING_STEPS),
            "eval_args": trainer_pb2.EvalArgs(num_steps=EVALUATION_STEPS),
        }
        trainer = Trainer(**training_kwargs)
        model_trainer = ModelTrainer(trainer=trainer,
                                     )
        return model_trainer
    except Exception as e:
        raise CensusConsumerException(e, sys) from e



import sys
import os
from tfx.components.base import executor_spec
from census_consumer_complaint_exception.exception import CensusConsumerException
from collections import namedtuple

ZIP_INPUT_DATASET_URL = "https://files.consumerfinance.gov/ccdb/complaints.csv.zip"
ZIP_CSV_EXTRACTOR_GEN_NAME = "ZIP_CSV_EXTRACTOR_GEN"
DataIngestion = namedtuple("DataIngestion", ["zip_example_gen"])
INPUT_BASE = os.path.join(os.getcwd(),"zip_to_csv")

from census_consumer_complaint_custom_component.example_gen import RemoteZipCsvExampleGen


def get_data_ingestion_components(url: str = ZIP_INPUT_DATASET_URL, input_base: str = INPUT_BASE) -> DataIngestion:
    """
    :param input_base:
    :param url:
    :param self:
    :return: List of tfx component
    """
    try:

        input_config = {

        }
        output_config = {

        }
        zip_example_gen = RemoteZipCsvExampleGen(
            zip_file_uri=url,
            input_base=input_base,
        )

        return DataIngestion(zip_example_gen=zip_example_gen)

    except Exception as e:
        raise CensusConsumerException(e, sys) from e



import csv
import datetime
import sys, os
import tensorflow as tf
import shutil
from census_consumer_complaint_exception.exception import CensusConsumerException
from urllib import request
import random
import numpy as np
from collections import namedtuple
from collections import OrderedDict
from zipfile import ZipFile

CSV_FILE_EXTENSION = ".csv"
TF_RECORD_FILE_EXTENSION = ".tfrecord"
ROW = 0


# defining function to convert value to appropriate data type which tf.Example accepts
def _bytes_feature(value):
    """
    created by: Avnish Yadav
    created on: 27/02/2022
    version: 1.0
    """
    try:
        value = bytes(value, encoding="utf-8")
        return tf.train.Feature(bytes_list=tf.train.BytesList(value=[value]))
    except Exception as e:
        raise Exception(CensusConsumerException(e, sys)) from e


def _float_feature(value):
    try:
        """
        created by: Avnish Yadav
        created on: 27/02/2022
        version: 1.0
        """
        return tf.train.Feature(float_list=tf.train.FloatList(value=[value]))
    except Exception as e:
        raise Exception(CensusConsumerException(e, sys)) from e


def _int64_feature(value):
    """
    created by: Avnish Yadav
    created on: 27/02/2022
    version: 1.0
    """
    try:
        return tf.train.Feature(int64_list=tf.train.Int64List(value=[value]))
    except Exception as e:
        raise Exception(CensusConsumerException(e, sys)) from e


def _convert_csv_file_to_tf_record_file(csv_file_path, tf_record_file_dir: str):
    try:
        n_row = 0
        tf_record_file_name = os.path.basename(csv_file_path).replace(CSV_FILE_EXTENSION, TF_RECORD_FILE_EXTENSION)
        tf_record_file_path=os.path.join(tf_record_file_dir,tf_record_file_name)
        tf_record_file_writer = tf.io.TFRecordWriter(tf_record_file_path)
        with open(csv_file_path) as csv_file:
            reader = csv.DictReader(csv_file, delimiter=",", quotechar='"')
            for row in reader:
                n_row += 1
                if n_row%100000==0:
                    print(n_row,"\n",datetime.datetime.now())
                    break
                example = tf.train.Example(features=tf.train.Features(
                    feature={
                        "product": _bytes_feature(row["Product"]),
                        "sub_product": _bytes_feature(row["Sub-product"]),
                        "issue": _bytes_feature(row["Issue"]),
                        "sub_issue": _bytes_feature(row["Sub-issue"]),
                        "state": _bytes_feature(row["State"]),
                        "zip_code": _bytes_feature((row["ZIP code"])),
                        "company": _bytes_feature(row["Company"]),
                        "company_response": _bytes_feature(row["Company response to consumer"]),
                        "consumer_complaint_narrative": _bytes_feature(row["Consumer complaint narrative"]),
                        "timely_response": _bytes_feature(row["Timely response?"]),
                        "consumer_disputed": _bytes_feature(row["Consumer disputed?"]),
                    }
                )
                )
            
                tf_record_file_writer.write(example.SerializeToString())  
        tf_record_file_writer.close()
    except Exception as e:
        raise Exception(CensusConsumerException(e, sys)) from e


def transform_csv_to_tf_record_file(csv_file_dir, tf_record_file_dir: str):
    """
    Description: This function accept csv file directory and converts all csv file into
    tfrecord file at tf_record_file_dir
    =============================================================================
    :param csv_file_dir: Dir path containing csv files
    :param tf_record_file_dir: Dir path to generated tf record files from csv files
    :return: dict with file_path and number of row
    {file_path:n_row}

    """
    try:

        csv_files = filter(lambda x: x.endswith(CSV_FILE_EXTENSION),
                           os.listdir(csv_file_dir))
        os.makedirs(tf_record_file_dir,exist_ok=True)
        print(csv_files,tf_record_file_dir)
        for file in csv_files:
            csv_file_path = os.path.join(csv_file_dir, file)
            _convert_csv_file_to_tf_record_file(csv_file_path, tf_record_file_dir)
            os.remove(csv_file_path)
    except Exception as e:
        raise (CensusConsumerException(e, sys)) from e


def extract_zip_file(zip_file_path: str, extract_dir: str, zip_file_read_mode: str = "r") -> None:
    """Extracts a zip file to a directory.

  Args:
  zip_file_path: The path to the zip file.
  output_dir: The directory to extract the zip file to.
  """
    try:
        print("Extracting file in ",extract_dir)
        os.makedirs(extract_dir, exist_ok=True)
        with ZipFile(zip_file_path, zip_file_read_mode) as zip_file:
            zip_file.extractall(extract_dir)
    except Exception as e:
        raise e


def parse_file(element, columns):
    global ROW
    ROW += 1

    """Read a line of CSV file and transform it into ordered dict"""
    for line in csv.reader([element], quotechar='"', delimiter=',', quoting=csv.QUOTE_ALL, skipinitialspace=True):

        if len(line) != len(columns):
            line = ["" for i in range(len(columns))]
        if ROW % 100000 == 0:
            print(ROW, datetime.datetime.now())
            print(line)
        return OrderedDict(zip(columns, line))


## code added by Avnish327030
def download_dataset(zip_file_uri: str, download_dir: str) -> str:
    """Downloads a dataset from a given uri and saves it to a Download directory.

  Args:
    zip_file_uri: The uri of the dataset to download.
    download_dir: The directory to save the dataset to.

  Returns:
    The path to the downloaded dataset.

  Raises:
    ValueError: If the dataset cannot be downloaded.
  """

    try:
        print("Downloading file in ",download_dir)
        # Creating download_dir if not exists
        if os.path.exists(download_dir):
            shutil.rmtree(download_dir)
        os.makedirs(download_dir, exist_ok=True)
        # Obtaining zip file path to download zip file
        zip_file_path = os.path.join(download_dir, os.path.basename(zip_file_uri))
        request.urlretrieve(zip_file_uri, zip_file_path)
        return zip_file_path
    except Exception as e:
        raise ValueError('Failed to download dataset from {}. {}'.format(zip_file_uri, e))



# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Avro based TFX example gen executor."""
import datetime
import os
from typing import Any, Dict

from absl import logging
import apache_beam as beam
import tensorflow as tf

from tfx.components.example_gen import utils
from tfx.components.example_gen.base_example_gen_executor import BaseExampleGenExecutor
from tfx.types import standard_component_specs
from zipfile import ZipFile
import csv
from collections import OrderedDict
from census_consumer_complaint_utils.utils import download_dataset

FILE_READ_FORMAT = "r"

COLUMNS = ['Date received', 'Product', 'Sub-product', 'Issue', 'Sub-issue', 'Consumer complaint narrative',
           'Company public response', 'Company', 'State', 'ZIP code', 'Tags', 'Consumer consent provided?',
           'Submitted via', 'Date sent to company', 'Company response to consumer', 'Timely response?',
           'Consumer disputed?', 'Complaint ID']

# COLUMNS = "pickup_community_area,fare,trip_start_month,trip_start_hour,trip_start_day,trip_start_timestamp,pickup_latitude,pickup_longitude,dropoff_latitude,dropoff_longitude,trip_miles,pickup_census_tract,dropoff_census_tract,payment_type,company,trip_seconds,dropoff_community_area,tips".split(
#     ",")
REMOTE_ZIP_FILE_URI = 'zip_file_uri'


def dict_to_example(element):
    print(element)
    utils.dict_to_example(element)


COLUMNS = None


def parse_file(element):
    global COLUMNS
    for line in csv.reader([element], quotechar='"', delimiter=',', quoting=csv.QUOTE_ALL, skipinitialspace=True):
        if COLUMNS is None:
            COLUMNS = line
            line = ["" for data in line]
        return OrderedDict(zip(COLUMNS, line))


@beam.ptransform_fn
@beam.typehints.with_input_types(beam.Pipeline)
@beam.typehints.with_output_types(tf.train.Example)
def _ZipToExample(  # pylint: disable=invalid-name
        pipeline: beam.Pipeline, exec_properties: Dict[str, Any],
        split_pattern: str) -> beam.pvalue.PCollection:
    """Read Avro files and transform to TF examples.

  Note that each input split will be transformed by this function separately.

  Args:
    pipeline: beam pipeline.
    exec_properties: A dict of execution properties.
      - input_base: input dir that contains Avro data.
    split_pattern: Split.pattern in Input config, glob relative file pattern
      that maps to input files with root directory given by input_base.

  Returns:
    PCollection of TF examples.
  """
    # directory to extract zip file
    input_base_uri = exec_properties[standard_component_specs.INPUT_BASE_KEY]

    # remote zip file uri to download zip file
    zip_file_uri = exec_properties[REMOTE_ZIP_FILE_URI]

    # downloading zip file from zip file uri into input_base_uri location
    zip_file_path = download_dataset(zip_file_uri, input_base_uri)

    # extracting zip file and deleteing zip file from directory
    with ZipFile(zip_file_path, FILE_READ_FORMAT) as zip_file:
        zip_file.extractall(path=os.path.join(input_base_uri))
    os.remove(zip_file_path)

    # obtain csv file path
    csv_file_path = os.path.join(input_base_uri, os.listdir(input_base_uri)[0])
    # comment the cdode
    # import pandas as pd
    # df = pd.read_csv(csv_file_path)
    # os.remove(csv_file_path)
    # df.iloc[:5000,:].to_csv(csv_file_path, index=None, header=True, mode="w")

    # uncomment the code
    return (pipeline
            | 'ReadCsvFile' >> beam.io.ReadFromText(csv_file_path)
            | 'ParseFile' >> beam.Map(parse_file)
            | "ToTFExample" >> beam.Map(utils.dict_to_example)
            )

    # utils.dict_to_example()
    # return (pipeline
    #         | 'ReadFromAvro' >> beam.io.ReadFromAvro(avro_pattern)
    #         | 'ToTFExample' >> beam.Map(utils.dict_to_example))


class Executor(BaseExampleGenExecutor):
    """TFX example gen executor for processing avro format.

  Data type conversion:
    integer types will be converted to tf.train.Feature with tf.train.Int64List.
    float types will be converted to tf.train.Feature with tf.train.FloatList.
    string types will be converted to tf.train.Feature with tf.train.BytesList
      and utf-8 encoding.

    Note that,
      Single value will be converted to a list of that single value.
      Missing value will be converted to empty tf.train.Feature().

    For details, check the dict_to_example function in example_gen.utils.


  Example usage:

    from tfx.components.base import executor_spec
    from tfx.components.example_gen.component import
    FileBasedExampleGen
    from tfx.components.example_gen.custom_executors import
    avro_executor

    example_gen = FileBasedExampleGen(
        input_base=avro_dir_path,
        custom_executor_spec=executor_spec.ExecutorClassSpec(
            avro_executor.Executor))
  """

    def GetInputSourceToExamplePTransform(self) -> beam.PTransform:
        """Returns PTransform for avro to TF examples."""
        return _ZipToExample



# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Generic TFX ImportExampleGen executor."""

import os
from typing import Any, Dict, Union

from absl import logging
import apache_beam as beam
import tensorflow as tf
from census_consumer_complaint_utils.utils import download_dataset, extract_zip_file, transform_csv_to_tf_record_file
from tfx.components.example_gen import base_example_gen_executor
from tfx.proto import example_gen_pb2
from tfx.types import standard_component_specs
from census_consumer_complaint_types.types import REMOTE_ZIP_FILE_URI_KEY


@beam.ptransform_fn
@beam.typehints.with_input_types(beam.Pipeline)
@beam.typehints.with_output_types(bytes)
def _ImportSerializedRecord(  # pylint: disable=invalid-name
        pipeline: beam.Pipeline, exec_properties: Dict[str, Any],
        split_pattern: str) -> beam.pvalue.PCollection:
    """Read TFRecord files to PCollection of records.

  Note that each input split will be transformed by this function separately.

  Args:
    pipeline: Beam pipeline.
    exec_properties: A dict of execution properties.
      - input_base: input dir that contains input data.
    split_pattern: Split.pattern in Input config, glob relative file pattern
      that maps to input files with root directory given by input_base.

  Returns:
    PCollection of records (tf.Example, tf.SequenceExample, or bytes).
  """
    input_base_uri = exec_properties[standard_component_specs.INPUT_BASE_KEY]
    input_split_pattern = os.path.join(input_base_uri, split_pattern)
    logging.info('Reading input TFRecord data %s.', input_split_pattern)

    # TODO(jyzhao): profile input examples.
    return (pipeline
            # TODO(jyzhao): support multiple input container format.
            | 'ReadFromTFRecord' >>
            beam.io.ReadFromTFRecord(file_pattern=input_split_pattern))


class Executor(base_example_gen_executor.BaseExampleGenExecutor):
    """Generic TFX import example gen executor."""

    def GetInputSourceToExamplePTransform(self) -> beam.PTransform:
        """Returns PTransform for importing records."""

        @beam.ptransform_fn
        @beam.typehints.with_input_types(beam.Pipeline)
        @beam.typehints.with_output_types(Union[tf.train.Example,
                                                tf.train.SequenceExample, bytes])
        def ImportRecord(pipeline: beam.Pipeline, exec_properties: Dict[str, Any],
                         split_pattern: str) -> beam.pvalue.PCollection:
            """PTransform to import records.

      The records are tf.train.Example, tf.train.SequenceExample,
      or serialized proto.

      Args:
        pipeline: Beam pipeline.
        exec_properties: A dict of execution properties.
          - input_base: input dir that contains input data.
        split_pattern: Split.pattern in Input config, glob relative file pattern
          that maps to input files with root directory given by input_base.

      Returns:
        PCollection of records (tf.Example, tf.SequenceExample, or bytes).
      """
            input_base_uri = os.path.join(exec_properties[standard_component_specs.INPUT_BASE_KEY])
            zip_file_uri = exec_properties[REMOTE_ZIP_FILE_URI_KEY]

            # downloading zip file from zip file uri into input_base_uri location
            zip_file_path = download_dataset(zip_file_uri, input_base_uri)
            extract_zip_file(zip_file_path, input_base_uri)
            os.remove(zip_file_path)
            transform_csv_to_tf_record_file(csv_file_dir=input_base_uri, tf_record_file_dir=input_base_uri)

            output_payload_format = exec_properties.get(
                standard_component_specs.OUTPUT_DATA_FORMAT_KEY)

            serialized_records = (
                    pipeline
                    # pylint: disable=no-value-for-parameter
                    | _ImportSerializedRecord(exec_properties, split_pattern))
            if output_payload_format == example_gen_pb2.PayloadFormat.FORMAT_PROTO:
                return serialized_records
            elif (output_payload_format ==
                  example_gen_pb2.PayloadFormat.FORMAT_TF_EXAMPLE):
                return (serialized_records
                        | 'ToTFExample' >> beam.Map(tf.train.Example.FromString))
            elif (output_payload_format ==
                  example_gen_pb2.PayloadFormat.FORMAT_TF_SEQUENCE_EXAMPLE):
                return (serialized_records
                        | 'ToTFSequenceExample' >> beam.Map(
                            tf.train.SequenceExample.FromString))

            raise ValueError('output_payload_format must be one of FORMAT_TF_EXAMPLE,'
                             ' FORMAT_TF_SEQUENCE_EXAMPLE or FORMAT_PROTO')

        return ImportRecord



import sys

from census_consumer_complaint_component.data_ingestion import get_data_ingestion_components
from census_consumer_complaint_component.data_validation import get_data_validation_components
from census_consumer_complaint_component.data_preprocessing import get_data_preprocessing_components
from census_consumer_complaint_exception.exception import CensusConsumerException
from census_consumer_complaint_component.model_trainer import get_model_trainer_component
from census_consumer_complaint_component.model_evaluation import get_model_evaluation_component
from census_consumer_complaint_component.model_pusher import get_model_pusher_component


def get_census_consumer_complaint_pipeline_component():
    try:

        pipeline_component = []

        # getting data ingestion component
        data_ingestion = get_data_ingestion_components()
        pipeline_component.append(data_ingestion.zip_example_gen)

        # getting data validation component
        data_validation = get_data_validation_components(data_ingestion=data_ingestion)
        pipeline_component.append(data_validation.statistic_gen)
        pipeline_component.append(data_validation.schema_gen)
        pipeline_component.append(data_validation.example_val)

        # getting data transformation component
        data_preprocessing = get_data_preprocessing_components(data_ingestion=data_ingestion,
                                                               data_validation=data_validation)

        pipeline_component.append(data_preprocessing.transformer)

        model_trainer = get_model_trainer_component(data_validation=data_validation,
                                                    data_preprocessing=data_preprocessing
                                                    )
        # returning pipeline component
        pipeline_component.append(model_trainer.trainer)

        model_analysis = get_model_evaluation_component(data_ingestion=data_ingestion,
                                                        trainer=model_trainer)

        pipeline_component.append(model_analysis.resolver)
        pipeline_component.append(model_analysis.evaluator)

        model_pusher = get_model_pusher_component(trainer=model_trainer, evaluator=model_analysis)
        pipeline_component.append(model_pusher.pusher)

        return pipeline_component

    except Exception as e:
        raise (CensusConsumerException(e, sys)) from e



from census_consumer_complaint_component.model_trainer import ModelTrainer
from census_consumer_complaint_component.model_evaluation import ModelEvaluation
from census_consumer_complaint_exception.exception import CensusConsumerException
from census_consumer_complaint_config.configuration import CensusConsumerConfiguration
import sys
from tfx.components import Pusher
from collections import namedtuple
from tfx.proto import pusher_pb2

ModelPusher = namedtuple("ModelPusher", ["pusher"])
config = CensusConsumerConfiguration()


def get_model_pusher_component(trainer: ModelTrainer, evaluator: ModelEvaluation) -> ModelPusher:
    try:
        pusher = Pusher(
            model=trainer.trainer.outputs['model'],
            model_blessing=evaluator.evaluator.outputs['blessing'],
            push_destination=pusher_pb2.PushDestination(
                filesystem=pusher_pb2.PushDestination.Filesystem(
                    base_directory=config.serving_model_dir
                )
            ),
        )
        print(f"save model dir:{config.serving_model_dir}")

        model_pusher = ModelPusher(pusher=pusher)
        return model_pusher

    except Exception as e:
        raise CensusConsumerException(e, sys) from e



import os
import datetime
from census_consumer_complaint_component.feature_engineering import  feature_engineering
from census_consumer_complaint_component.model_training import trainer

ROOT_DIR = os.path.join("census_consumer_complaint_data")
PIPELINE_NAME = "census_consumer_complaint"
PIPELINE_ARTIFACT = "artifact"
META_DATA_SQLITE_FILE_DIR = "meta_data"
SQLITE_FILE_NAME = "meta_data.db"
LOG_DIR = 'logs'
SERVING_MODEL_DIR = "saved_models"
DAYS = 1  # scheduled interval day
SCHEDULED_INTERVAL = datetime.timedelta(days=DAYS)
START_DATE = datetime.datetime(2022, 3, 22)
TRANSFORM_MODULE_FILE = feature_engineering.__file__
    # os.path.join(WORK_DIR, "census_consumer_complaint_component", "feature_engineering",
    #                                  "feature_engineering.py")
TRAINER_MODULE_FILE = trainer.__file__
    # os.path.join(WORK_DIR, "census_consumer_complaint_component", "model_training",
    #                                "trainer.py")


class CensusConsumerConfiguration:

    def __init__(self):
        self.root_dir = os.path.join(ROOT_DIR)
        self.pipeline_name = PIPELINE_NAME
        self.pipeline_root = os.path.join(ROOT_DIR, PIPELINE_NAME, PIPELINE_ARTIFACT)
        self.metadata_path = os.path.join(ROOT_DIR, META_DATA_SQLITE_FILE_DIR, SQLITE_FILE_NAME)
        self.log_dir = os.path.join(ROOT_DIR, LOG_DIR)
        self.serving_model_dir = os.path.join(ROOT_DIR, SERVING_MODEL_DIR)
        self.scheduled_interval = SCHEDULED_INTERVAL
        self.start_date = START_DATE
        self.transform_module_file_path = TRANSFORM_MODULE_FILE
        self.trainer_module_file_path = TRAINER_MODULE_FILE



import sys
from census_consumer_complaint_exception.exception import CensusConsumerException
import tensorflow as tf
import tensorflow_transform as tft

coulmns = ['Company', 'Company public response', 'Company response to consumer',
           'Complaint ID', 'Consumer complaint narrative', 'Consumer consent provided?',
           'Consumer disputed?', 'Date received', 'Date sent to company', 'Issue',
           'Product', 'State', 'Sub-issue',
           'Sub-product', 'Submitted via', 'Tags', 'Timely response?', 'ZIP code']
ONE_HOT_FEATURES = {
    "Product": 11,
    "Sub-product": 45,
    "Company response to consumer": 5,
    "State": 60,
    "Issue": 90
}

LABEL_KEY = "Consumer disputed?"

BUCKET_FEATURES = {
    "ZIP code": 10
}

TEXT_FEATURES = {
    "Consumer complaint narrative": None

}


def transformed_name(key):
    try:
        return key + "_xf"
    except Exception as e:
        raise CensusConsumerException(e, sys) from e


def fill_in_missing(x):
    try:
        is_updated = False
        default_value = "" if x.dtype == tf.string else 0
        print(type(x))
        if type(x) == tf.SparseTensor:
            is_updated = True
            x = tf.sparse.to_dense(
                tf.SparseTensor(
                    x.indices, x.values, [x.dense_shape[0], 1]),
                default_value
            )
        print(f"Column: {x} has been updated: {is_updated}")

        return tf.squeeze(x, axis=1)
    except Exception as e:
        raise CensusConsumerException(e, sys) from e


def convert_num_to_one_hot(label_tensor, num_labels=2):
    try:
        one_hot_tensor = tf.one_hot(label_tensor, num_labels)
        return tf.reshape(one_hot_tensor, [-1, num_labels])
    except Exception as e:
        raise CensusConsumerException(e, sys) from e


@tf.function
def cast_target_column_to_integer(x):
    x = tf.strings.lower(fill_in_missing(x))
    return tf.cast(tf.equal(x, 'no'), tf.int64)


@tf.function
def convert_zip_code(zip_code):
    try:
        if zip_code == "":
            zip_code = "00000"
        zip_code = tf.strings.regex_replace(zip_code, r'X{0,5}', "0")
        zip_code = tf.strings.to_number(zip_code, tf.float32)
        return zip_code
    except Exception as e:
        print(e)
        raise CensusConsumerException(e, sys) from e


def preprocessing_fn(inputs):
    try:
        print(f"Columns: {inputs.keys()}")
        outputs = {}
        for key in ONE_HOT_FEATURES.keys():
            dim = ONE_HOT_FEATURES[key]
            index = tft.compute_and_apply_vocabulary(
                fill_in_missing(inputs[key]), top_k=dim + 1)
            outputs[transformed_name(key)] = convert_num_to_one_hot(index, num_labels=dim + 1)

        # for key, bucket_count in BUCKET_FEATURES.items():
        #     temp_feature = tft.bucketize(
        #         tft.bucketize(convert_zip_code(fill_in_missing(inputs[key])),
        #                       num_buckets=5,
        #                       )  # , num_buckets=10
        #     )

        # outputs[transformed_name(key)] = convert_num_to_one_hot(temp_feature, num_labels=bucket_count + 1)

        for key in TEXT_FEATURES.keys():
            outputs[transformed_name(key)] = fill_in_missing(inputs[key])
        # converting label categorical column into integer
        outputs[transformed_name(LABEL_KEY)] = inputs[LABEL_KEY]
        return outputs
    except Exception as e:
        raise CensusConsumerException(e, sys) from e


from census_consumer_complaint_exception.exception import CensusConsumerException
from census_consumer_complaint_component.data_ingestion import DataIngestion
from census_consumer_complaint_component.model_trainer import ModelTrainer
import sys, os

from tfx.components import Evaluator
from tfx.v1.dsl import Resolver, Channel
from tfx.v1.types.standard_artifacts import Model, ModelBlessing

from tfx.v1.dsl import experimental

import tensorflow_model_analysis as tfma
from census_consumer_complaint_component.feature_engineering.feature_engineering import LABEL_KEY
from collections import namedtuple

ModelEvaluation = namedtuple("ModelEvaluation", ["evaluator", "resolver"])


def get_model_evaluation_component(data_ingestion: DataIngestion, trainer: ModelTrainer):
    try:
        eval_config = tfma.EvalConfig(
            model_specs=[
                tfma.ModelSpec(
                    signature_name="serving_default",
                    label_key=LABEL_KEY,

                )
            ],
            slicing_specs=[tfma.SlicingSpec(), tfma.SlicingSpec(feature_keys=["product"])],
            metrics_specs=[
                tfma.MetricsSpec(
                    metrics=[
                        tfma.MetricConfig(
                            class_name="BinaryAccuracy",
                            threshold=tfma.MetricThreshold(
                                value_threshold=tfma.GenericValueThreshold(
                                    lower_bound={"value": 0.5}
                                ),
                                change_threshold=tfma.GenericChangeThreshold(
                                    direction=tfma.MetricDirection.HIGHER_IS_BETTER,
                                    absolute={"value": -1e-10},
                                ),
                            ),
                        ),
                        tfma.MetricConfig(class_name="Precision"),
                        tfma.MetricConfig(class_name="Recall"),
                        tfma.MetricConfig(class_name="ExampleCount"),
                        tfma.MetricConfig(class_name="AUC"),
                    ],
                )
            ],
        )
        model_resolver = Resolver(
            strategy_class=experimental.LatestBlessedModelStrategy,
            model=Channel(type=Model),
            model_blessing=Channel(type=ModelBlessing),
        )
        evaluator = Evaluator(
            examples=data_ingestion.zip_example_gen.outputs['examples'],
            model=trainer.trainer.outputs['model'],
            baseline_model=model_resolver.outputs["model"],
            eval_config=eval_config,
        )
        evaluator = Evaluator(
            examples=data_ingestion.zip_example_gen.outputs['examples'],
            model=trainer.trainer.outputs['model'],
            eval_config=eval_config,
            baseline_model=model_resolver.outputs['model'],

        )
        return ModelEvaluation(resolver=model_resolver, evaluator=evaluator)

    except Exception as e:
        raise CensusConsumerException(e, sys) from e



from setuptools import setup, find_packages

setup(
    name="airflow-census",
    license="MIT",
    version="0.0.0",
    description="""
    This project is created to provide a simple way to
    ingest data from the Census API.
    https://files.consumerfinance.gov/ccdb/complaints.csv.zip

    Project provide prebuilt airflow DAG pipleine designed using TFX.
    
    how to use this library

    ```
    from census_consumer_complaint_orchestrator.airflow_orchestrator import get_airflow_dag_pipeline
    dag = get_airflow_dag_pipeline()
    ```


    """,
    author="Avnish Yadav",
    packages=find_packages(),
    install_requires=['tfx==1.6.1', 'apache-beam[interactive]', 'apache-airflow']
)



from census_consumer_complaint_config.configuration import CensusConsumerConfiguration
from census_consumer_complaint_exception.exception import CensusConsumerException
from tfx.orchestration.experimental.interactive.interactive_context import InteractiveContext
from tfx.orchestration.metadata import sqlite_metadata_connection_config
import sys,os


class CensusConsumerInteractiveContext(CensusConsumerConfiguration):

    def __init__(self, *args, **kwargs):
        try:
            super(CensusConsumerInteractiveContext, self).__init__()
            self.interactive_context = None
        except Exception as e:
            raise (CensusConsumerException(e, sys)) from e

    def get_interactive_context(self):
        try:

            if self.interactive_context is None:
                self.interactive_context = InteractiveContext(
                    pipeline_name=self.pipeline_name,
                    pipeline_root=self.pipeline_root,
                    metadata_connection_config=sqlite_metadata_connection_config(self.metadata_path)
                )
            return self.interactive_context
        except Exception as e:
            raise (CensusConsumerException(e, sys)) from e



from typing import Optional, Union

from tfx import types
from tfx.components.example_gen import driver
from tfx.components.example_gen import utils
from tfx.dsl.components.base import base_beam_component
from tfx.dsl.components.base import base_beam_executor
from tfx.dsl.components.base import executor_spec
from tfx.orchestration import data_types
from tfx.proto import example_gen_pb2
from tfx.proto import range_config_pb2
from tfx.types import standard_artifacts
from census_consumer_complaint_types.types import RemoteZipFileBasedExampleGenSpec
from tfx.components import FileBasedExampleGen


class RemoteZipFileBasedExampleGen(base_beam_component.BaseBeamComponent):
    """A TFX component to ingest examples from a file system.

  The FileBasedExampleGen component is an API for getting file-based records
  into TFX pipelines. It consumes external files to generate examples which will
  be used by other internal components like StatisticsGen or Trainers.  The
  component will also convert the input data into
  [tf.record](https://www.tensorflow.org/tutorials/load_data/tf_records)
  and generate train and eval example splits for downstream components.

  ## Example
  ```
  _taxi_root = os.path.join(os.environ['HOME'], 'taxi')
  _data_root = os.path.join(_taxi_root, 'data', 'simple')
  _zip_uri = "https://xyz//abz.csv.zip"
  # Brings data into the pipeline or otherwise joins/converts training data.
  example_gen = RemoteZipFileBasedExample(input_base=_data_root,zip_file_uri="")
  ```

  Component `outputs` contains:
   - `examples`: Channel of type `standard_artifacts.Examples` for output train
                 and eval examples.
  """

    SPEC_CLASS = RemoteZipFileBasedExampleGenSpec

    # EXECUTOR_SPEC should be overridden by subclasses.
    EXECUTOR_SPEC = executor_spec.BeamExecutorSpec(
        base_beam_executor.BaseBeamExecutor)
    DRIVER_CLASS = driver.FileBasedDriver

    def __init__(
            self,
            input_base: Optional[str] = None,
            zip_file_uri: Optional[str] = None,
            input_config: Optional[Union[example_gen_pb2.Input,
                                         data_types.RuntimeParameter]] = None,
            output_config: Optional[Union[example_gen_pb2.Output,
                                          data_types.RuntimeParameter]] = None,
            custom_config: Optional[Union[example_gen_pb2.CustomConfig,
                                          data_types.RuntimeParameter]] = None,
            range_config: Optional[Union[range_config_pb2.RangeConfig,
                                         data_types.RuntimeParameter]] = None,
            output_data_format: Optional[int] = example_gen_pb2.FORMAT_TF_EXAMPLE,
            output_file_format: Optional[int] = example_gen_pb2.FORMAT_TFRECORDS_GZIP,
            custom_executor_spec: Optional[executor_spec.ExecutorSpec] = None):
        """Construct a FileBasedExampleGen component.

    Args:
      input_base: an extract directory containing the CSV files after extraction of downloaded zip file.
      zip_file_uri: Remote Zip file uri to download compressed zip csv file
      input_config: An
        [`example_gen_pb2.Input`](https://github.com/tensorflow/tfx/blob/master/tfx/proto/example_gen.proto)
          instance, providing input configuration. If unset, input files will be
          treated as a single split.
      output_config: An example_gen_pb2.Output instance, providing the output
        configuration. If unset, default splits will be 'train' and
        'eval' with size 2:1.
      custom_config: An optional example_gen_pb2.CustomConfig instance,
        providing custom configuration for executor.
      range_config: An optional range_config_pb2.RangeConfig instance,
        specifying the range of span values to consider. If unset, driver will
        default to searching for latest span with no restrictions.
      output_data_format: Payload format of generated data in output artifact,
        one of example_gen_pb2.PayloadFormat enum.
      output_file_format: File format of generated data in output artifact,
        one of example_gen_pb2.FileFormat enum.
      custom_executor_spec: Optional custom executor spec overriding the default
        executor spec specified in the component attribute.
    """
        # Configure inputs and outputs.
        input_config = input_config or utils.make_default_input_config()
        output_config = output_config or utils.make_default_output_config(
            input_config)
        example_artifacts = types.Channel(type=standard_artifacts.Examples)
        spec = RemoteZipFileBasedExampleGenSpec(
            input_base=input_base,
            zip_file_uri=zip_file_uri,
            input_config=input_config,
            output_config=output_config,
            custom_config=custom_config,
            range_config=range_config,
            output_data_format=output_data_format,
            output_file_format=output_file_format,
            examples=example_artifacts)
        super().__init__(spec=spec, custom_executor_spec=custom_executor_spec)
